#ifndef _SENSFUSION_MATRIX_SUPPORT
#define _SENSFUSION_MATRIX_SUPPORT



/*
 * Copyright (c) 2011 LAAS/CNRS                       --  Oct 2011
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * Description: low-pass, Kalman and other Bayesian filters for perception
 *  
 *  
 *  
 *  Creation date : Jan 19 22:35:29 2011 
 *  Author: HE Wuwei
 *  LAAS-CNRS, Toulouse FRANCE
 */

/*
 * Matrix and basic numerical support for the Unscented Kalman Filter
 * (Basic, Scaled and Spherical Simplex)
 * 
 * The matrix part is copied and modified from the Bayes++ library by 
 * Copyright (c) 2002 Michael Stevens
 * And modified by HE Wuwei wuwei.he@laas.fr
 * Nov. 2011
 * 
 */

/*
 * Matrix support functions for filter classes
 *  Members of the sensFusion_matrix namespace are used in the
 *  interface of Bayesian_filter and for internal operations.
 *  Be aware! These functions and their implemenation are more likely to change
 *  then those in Bayesian_filter.
 */

/* Filter Matrix Namespace */
namespace sensFusion_matrix
{


/*
 * Assertion support
 */
#ifndef NDEBUG
void assert_isPSD (const SymMatrix &M);
#else
inline void assert_isPSD (const SymMatrix &M) {}
#endif

/*
 * Local support functions
 */
bool isPSD (const SymMatrix &M);
bool isSymmetric (const Matrix &M);
void forceSymmetric (Matrix &M, bool bUpperToLower = false);

/*
 * UdU' and LdL' and UU' Cholesky Factorisation and function
 * Very important to manipulate PD and PSD matrices
 *
 * Return values:
 *  Many algorithms return a value_type which is a reciprocal condition number
 *  These values are documented for each algorithm and are important way to
 *  determine the validity of the results
 */
Vec::value_type UdUrcond (const Vec& d);
RowMatrix::value_type UdUrcond (const RowMatrix& UD);
RowMatrix::value_type UdUrcond (const RowMatrix& UD, std::size_t n);
UTriMatrix::value_type UCrcond (const UTriMatrix& UC);
SymMatrix::value_type UdUdet (const SymMatrix& UD);

// In-place factorisations
RowMatrix::value_type UdUfactor_variant1 (RowMatrix& M, std::size_t n);
RowMatrix::value_type UdUfactor_variant2 (RowMatrix& M, std::size_t n);
inline RowMatrix::value_type UdUfactor (RowMatrix& M, std::size_t n)
{	return UdUfactor_variant2(M,n);
}
LTriMatrix::value_type LdLfactor (LTriMatrix& M, std::size_t n);
UTriMatrix::value_type UCfactor (UTriMatrix& M, std::size_t n);

// Copy factorisations
RowMatrix::value_type UdUfactor (RowMatrix& UD, const SymMatrix& M);
LTriMatrix::value_type LdLfactor (LTriMatrix& LD, const SymMatrix& M);
UTriMatrix::value_type UCfactor (UTriMatrix& UC, const SymMatrix& M);

// Factor manipulations
bool UdUinverse (RowMatrix& UD);
bool UTinverse (UTriMatrix& U);
void UdUrecompose_transpose (RowMatrix& M);
void UdUrecompose (RowMatrix& M);
void UdUrecompose (SymMatrix& X, const RowMatrix& M);
void UdUfromUCholesky (RowMatrix& U);
void UdUseperate (RowMatrix& U, Vec& d, const RowMatrix& UD);
void Lzero (RowMatrix& M);
void Uzero (RowMatrix& M);

/*
 * Functions using UdU factorisation:
 *  inverse of Positive Definate matrix returning rcond
 */
SymMatrix::value_type UdUinversePDignoreInfinity (SymMatrix& M);
SymMatrix::value_type UdUinversePD (SymMatrix& M);
SymMatrix::value_type UdUinversePD (SymMatrix& M, SymMatrix::value_type& detM);
SymMatrix::value_type UdUinversePD (SymMatrix& MI, const SymMatrix& M);
SymMatrix::value_type UdUinversePD (SymMatrix& MI, SymMatrix::value_type& detM, const SymMatrix& M);


}//namespace

#endif
