#ifndef _SENSFUSION_EXCEPTION
#define _SENSFUSION_EXCEPTION

/*
 * Copyright (c) 2011 LAAS/CNRS                       --  Oct 2011
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * Description: low-pass, Kalman and other Bayesian filters for perception
 *  
 *  
 *  
 *  Creation date : Jan 19 22:35:29 2011 
 *  Author: HE Wuwei
 *  LAAS-CNRS, Toulouse FRANCE
 */


/*
 * Matrix and basic numerical support for the Unscented Kalman Filter
 * (Basic, Scaled and Spherical Simplex)
 * 
 * The matrix part is copied (and modified) from the Bayes++ library by 
 * Copyright (c) 2002 Michael Stevens
 * And modified by HE Wuwei wuwei.he@laas.fr
 * Nov. 2011
 * 
 */

/*
 * Exception types: Exception heirarchy for filtering 
 */
 
// Common headers required for declerations
#include <exception>

/* Filter namespace */
namespace sensFusion_filters
{


class Filter_exception : virtual public std::exception
/*
 *	Base class for all exception produced by filter heirachy
 */
{
public:
	const char *what() const throw()
	{	return error_description;
	}
protected:
	Filter_exception (const char* description)
	{	error_description = description;
	}
private:
	const char* error_description;
};

class Logic_exception : virtual public Filter_exception
/*
 * Logic Exception
 */
{
public:
	Logic_exception (const char* description) :
		Filter_exception (description)
	{}
};

class Numeric_exception : virtual public Filter_exception
/*
 * Numeric Exception
 */
{
public:
	Numeric_exception (const char* description) :
		Filter_exception (description)
	{}
};


}//namespace
#endif
